# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .trace_filter_write import TraceFilterWrite
from .user_defined_metric_python_code_write import UserDefinedMetricPythonCodeWrite


class AutomationRuleEvaluatorUserDefinedMetricPythonWrite(UniversalBaseModel):
    filters: typing.Optional[typing.List[TraceFilterWrite]] = None
    code: typing.Optional[UserDefinedMetricPythonCodeWrite] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
