# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .audio_url import AudioUrl
from .image_url import ImageUrl
from .video_url import VideoUrl


class LlmAsJudgeMessageContent(UniversalBaseModel):
    type: str
    text: typing.Optional[str] = None
    image_url: typing.Optional[ImageUrl] = None
    video_url: typing.Optional[VideoUrl] = None
    audio_url: typing.Optional[AudioUrl] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
