# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .alert_trigger_write import AlertTriggerWrite
from .alert_write_alert_type import AlertWriteAlertType
from .webhook_write import WebhookWrite


class AlertWrite(UniversalBaseModel):
    id: typing.Optional[str] = None
    name: typing.Optional[str] = None
    enabled: typing.Optional[bool] = None
    alert_type: typing.Optional[AlertWriteAlertType] = None
    metadata: typing.Optional[typing.Dict[str, str]] = None
    webhook: WebhookWrite
    triggers: typing.Optional[typing.List[AlertTriggerWrite]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
